from __future__ import annotations

from typing import Any

from _case import _camelize
from _case import _to_camel_case


def to_camel_case(snake_case_string: str) -> str:
    """Converts some_thing to someThing."""
    return str(_to_camel_case(snake_case_string))


def camelize(d: dict[str, Any]) -> dict[str, Any]:
    """Converts {'some_thing': {'having_value': 'like_so'}} to {'someThing': {'havingValue': 'like_so'}}."""
    return _camelize(d)



"""
YET ANOTHER STRING CASE CONVERTER
"""
from importlib.metadata import PackageNotFoundError

try:
    import importlib.metadata as importlib_metadata
except ModuleNotFoundError:
    import importlib_metadata

try:
    __version__ = importlib_metadata.version(__name__)
except PackageNotFoundError:
    __version__ = "dev"



from typing import Any
from typing import Callable
from typing import MutableMapping
from typing import MutableSequence
from typing import TypeVar
from typing import Union

T = TypeVar("T", bound=Union[MutableMapping, MutableSequence, Any])


def apply(f: Callable[[str], str], d: T) -> T:
    q = [d]
    while q:
        o = q.pop()
        if isinstance(o, dict):
            for k in [*o]:
                v = o[f(k) if isinstance(k, str) else k] = o.pop(k)
                if isinstance(v, (dict, list)):
                    q.append(v)
        if isinstance(o, list):
            q.extend(o)
    return d



import pytest

from yascc.camelcase import decamelize
from yascc.camelcase import to_snake_case

from .django_rest_framework_camelcase import underscoreize


def test_to_snake_case():
    assert to_snake_case("testString") == "test_string"


def test_decamelize_dict(camelized_dict: dict, decamelized_dict: dict) -> None:
    got = decamelize(camelized_dict)
    assert got == decamelized_dict


@pytest.mark.parametrize(
    "arg,expected",
    [
        (None, None),
        ([None], [None]),
        ([{None: None}], [{None: None}]),
        ([{"someValue": None}], [{"some_value": None}]),
    ],
)
def test_decamelize(arg, expected):
    assert decamelize(arg) == expected


@pytest.mark.parametrize("impl", [decamelize, underscoreize], ids=["c", "drf"])
def test_decamelize_list_benchmark(benchmark, camelized_list_of_dict, impl):
    benchmark(impl, camelized_list_of_dict)


@pytest.mark.parametrize("impl", [decamelize, underscoreize], ids=["c", "drf"])
def test_decamelize_benchmark(benchmark, camelized_dict, impl):
    benchmark(impl, camelized_dict)


def test_decamelize_nested_dict(camelized_nested_dict, decamelized_nested_dict) -> None:
    assert decamelize(camelized_nested_dict) == decamelized_nested_dict


def test_decamelize_list_of_dict(camelized_list_of_dict, decamelized_list_of_dict) -> None:
    assert decamelize(camelized_list_of_dict) == decamelized_list_of_dict



from __future__ import annotations

from typing import Any

from _case import _decamelize
from _case import _to_snake_case


def to_snake_case(src: str) -> str:
    return _to_snake_case(src)


def decamelize(d: dict[str, Any] | list[Any]) -> dict[str, Any] | list[Any]:
    return _decamelize(d)
